@@ -35,7 +35,7 @@ import (
 	"go.opentelemetry.io/otel/sdk/resource"
 	sdktrace "go.opentelemetry.io/otel/sdk/trace"
 	"go.opentelemetry.io/otel/sdk/trace/tracetest"
-	semconv "go.opentelemetry.io/otel/semconv/v1.7.0"
+	semconv "go.opentelemetry.io/otel/semconv/v1.17.0"
 	"go.opentelemetry.io/otel/trace"
 )
 
@@ -119,7 +119,7 @@ func TestExporterExportSpan(t *testing.T) {
 	tp := sdktrace.NewTracerProvider(
 		sdktrace.WithBatcher(exp),
 		sdktrace.WithResource(resource.NewSchemaless(
-			semconv.ServiceNameKey.String(serviceName),
+			semconv.ServiceName(serviceName),
 			attribute.String(tagKey, tagVal),
 		)),
 	)
@@ -146,7 +146,7 @@ func TestExporterExportSpan(t *testing.T) {
 	assert.Equal(t, tagVal, uploadedBatch.GetProcess().GetTags()[0].GetVStr())
 }
 
-func Test_spanSnapshotToThrift(t *testing.T) {
+func TestSpanSnapshotToThrift(t *testing.T) {
 	now := time.Now()
 	traceID, _ := trace.TraceIDFromHex("0102030405060708090a0b0c0d0e0f10")
 	spanID, _ := trace.SpanIDFromHex("0102030405060708")
@@ -158,7 +158,7 @@ func Test_spanSnapshotToThrift(t *testing.T) {
 	eventNameValue := "event-test"
 	eventDropped := int64(10)
 	keyValue := "value"
-	statusCodeValue := int64(1)
+	statusCodeValue := "ERROR"
 	doubleValue := 123.456
 	intValue := int64(123)
 	boolTrue := true
@@ -203,7 +203,7 @@ func Test_spanSnapshotToThrift(t *testing.T) {
 					{Key: keyError, VType: gen.TagType_BOOL, VBool: &boolTrue},
 					{Key: keyInstrumentationLibraryName, VType: gen.TagType_STRING, VStr: &instrLibName},
 					{Key: keyInstrumentationLibraryVersion, VType: gen.TagType_STRING, VStr: &instrLibVersion},
-					{Key: keyStatusCode, VType: gen.TagType_LONG, VLong: &statusCodeValue},
+					{Key: keyStatusCode, VType: gen.TagType_STRING, VStr: &statusCodeValue},
 					// Should not have a status message because it was unset
 					{Key: keySpanKind, VType: gen.TagType_STRING, VStr: &spanKind},
 				},
@@ -264,7 +264,7 @@ func Test_spanSnapshotToThrift(t *testing.T) {
 					{Key: keyError, VType: gen.TagType_BOOL, VBool: &boolTrue},
 					{Key: keyInstrumentationLibraryName, VType: gen.TagType_STRING, VStr: &instrLibName},
 					{Key: keyInstrumentationLibraryVersion, VType: gen.TagType_STRING, VStr: &instrLibVersion},
-					{Key: keyStatusCode, VType: gen.TagType_LONG, VLong: &statusCodeValue},
+					{Key: keyStatusCode, VType: gen.TagType_STRING, VStr: &statusCodeValue},
 					{Key: keyStatusMessage, VType: gen.TagType_STRING, VStr: &statusMessage},
 					{Key: keySpanKind, VType: gen.TagType_STRING, VStr: &spanKind},
 				},
@@ -376,7 +376,7 @@ func Test_spanSnapshotToThrift(t *testing.T) {
 				Resource: resource.NewSchemaless(
 					attribute.String("rk1", rv1),
 					attribute.Int64("rk2", rv2),
-					semconv.ServiceNameKey.String("service name"),
+					semconv.ServiceName("service name"),
 				),
 				Status: sdktrace.Status{
 					Code:        codes.Unset,
@@ -453,7 +453,7 @@ func TestExporterExportSpansHonorsCancel(t *testing.T) {
 		{
 			Name: "s1",
 			Resource: resource.NewSchemaless(
-				semconv.ServiceNameKey.String("name"),
+				semconv.ServiceName("name"),
 				attribute.Key("r1").String("v1"),
 			),
 			StartTime: now,
@@ -462,7 +462,7 @@ func TestExporterExportSpansHonorsCancel(t *testing.T) {
 		{
 			Name: "s2",
 			Resource: resource.NewSchemaless(
-				semconv.ServiceNameKey.String("name"),
+				semconv.ServiceName("name"),
 				attribute.Key("r2").String("v2"),
 			),
 			StartTime: now,
@@ -483,7 +483,7 @@ func TestExporterExportSpansHonorsTimeout(t *testing.T) {
 		{
 			Name: "s1",
 			Resource: resource.NewSchemaless(
-				semconv.ServiceNameKey.String("name"),
+				semconv.ServiceName("name"),
 				attribute.Key("r1").String("v1"),
 			),
 			StartTime: now,
@@ -492,7 +492,7 @@ func TestExporterExportSpansHonorsTimeout(t *testing.T) {
 		{
 			Name: "s2",
 			Resource: resource.NewSchemaless(
-				semconv.ServiceNameKey.String("name"),
+				semconv.ServiceName("name"),
 				attribute.Key("r2").String("v2"),
 			),
 			StartTime: now,
@@ -530,7 +530,7 @@ func TestJaegerBatchList(t *testing.T) {
 				tracetest.SpanStub{
 					Name: "s1",
 					Resource: resource.NewSchemaless(
-						semconv.ServiceNameKey.String("name"),
+						semconv.ServiceName("name"),
 						attribute.Key("r1").String("v1"),
 					),
 					StartTime: now,
@@ -564,7 +564,7 @@ func TestJaegerBatchList(t *testing.T) {
 				{
 					Name: "s1",
 					Resource: resource.NewSchemaless(
-						semconv.ServiceNameKey.String("name"),
+						semconv.ServiceName("name"),
 						attribute.Key("r1").String("v1"),
 					),
 					StartTime: now,
@@ -573,7 +573,7 @@ func TestJaegerBatchList(t *testing.T) {
 				{
 					Name: "s2",
 					Resource: resource.NewSchemaless(
-						semconv.ServiceNameKey.String("name"),
+						semconv.ServiceName("name"),
 						attribute.Key("r1").String("v1"),
 					),
 					StartTime: now,
@@ -582,7 +582,7 @@ func TestJaegerBatchList(t *testing.T) {
 				{
 					Name: "s3",
 					Resource: resource.NewSchemaless(
-						semconv.ServiceNameKey.String("name"),
+						semconv.ServiceName("name"),
 						attribute.Key("r2").String("v2"),
 					),
 					StartTime: now,
@@ -689,7 +689,7 @@ func TestProcess(t *testing.T) {
 		{
 			name: "resources contain service name",
 			res: resource.NewSchemaless(
-				semconv.ServiceNameKey.String("service name"),
+				semconv.ServiceName("service name"),
 				attribute.Key("r1").String("v1"),
 			),
 			defaultServiceName: "default service name",
