@@ -32,17 +32,98 @@ import (
 	"go.opentelemetry.io/otel/sdk/resource"
 	tracesdk "go.opentelemetry.io/otel/sdk/trace"
 	"go.opentelemetry.io/otel/sdk/trace/tracetest"
-	semconv "go.opentelemetry.io/otel/semconv/v1.7.0"
+	semconv "go.opentelemetry.io/otel/semconv/v1.17.0"
 	"go.opentelemetry.io/otel/trace"
 )
 
 func TestModelConversion(t *testing.T) {
-	resource := resource.NewSchemaless(
-		semconv.ServiceNameKey.String("model-test"),
+	res := resource.NewSchemaless(
+		semconv.ServiceName("model-test"),
+		semconv.ServiceVersion("0.1.0"),
+		attribute.Int64("resource-attr1", 42),
+		attribute.IntSlice("resource-attr2", []int{0, 1, 2}),
 	)
 
 	inputBatch := tracetest.SpanStubs{
-		// typical span data
+		// typical span data with UNSET status
+		{
+			SpanContext: trace.NewSpanContext(trace.SpanContextConfig{
+				TraceID: trace.TraceID{0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F},
+				SpanID:  trace.SpanID{0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8},
+			}),
+			Parent: trace.NewSpanContext(trace.SpanContextConfig{
+				TraceID: trace.TraceID{0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F},
+				SpanID:  trace.SpanID{0x3F, 0x3E, 0x3D, 0x3C, 0x3B, 0x3A, 0x39, 0x38},
+			}),
+			SpanKind:  trace.SpanKindServer,
+			Name:      "foo",
+			StartTime: time.Date(2020, time.March, 11, 19, 24, 0, 0, time.UTC),
+			EndTime:   time.Date(2020, time.March, 11, 19, 25, 0, 0, time.UTC),
+			Attributes: []attribute.KeyValue{
+				attribute.Int64("attr1", 42),
+				attribute.String("attr2", "bar"),
+				attribute.IntSlice("attr3", []int{0, 1, 2}),
+			},
+			Events: []tracesdk.Event{
+				{
+					Time: time.Date(2020, time.March, 11, 19, 24, 30, 0, time.UTC),
+					Name: "ev1",
+					Attributes: []attribute.KeyValue{
+						attribute.Int64("eventattr1", 123),
+					},
+				},
+				{
+					Time:       time.Date(2020, time.March, 11, 19, 24, 45, 0, time.UTC),
+					Name:       "ev2",
+					Attributes: nil,
+				},
+			},
+			Status: tracesdk.Status{
+				Code:        codes.Unset,
+				Description: "",
+			},
+			Resource: res,
+		},
+		// typical span data with OK status
+		{
+			SpanContext: trace.NewSpanContext(trace.SpanContextConfig{
+				TraceID: trace.TraceID{0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F},
+				SpanID:  trace.SpanID{0xFF, 0xFE, 0xFD, 0xFC, 0xFB, 0xFA, 0xF9, 0xF8},
+			}),
+			Parent: trace.NewSpanContext(trace.SpanContextConfig{
+				TraceID: trace.TraceID{0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F},
+				SpanID:  trace.SpanID{0x3F, 0x3E, 0x3D, 0x3C, 0x3B, 0x3A, 0x39, 0x38},
+			}),
+			SpanKind:  trace.SpanKindServer,
+			Name:      "foo",
+			StartTime: time.Date(2020, time.March, 11, 19, 24, 0, 0, time.UTC),
+			EndTime:   time.Date(2020, time.March, 11, 19, 25, 0, 0, time.UTC),
+			Attributes: []attribute.KeyValue{
+				attribute.Int64("attr1", 42),
+				attribute.String("attr2", "bar"),
+				attribute.IntSlice("attr3", []int{0, 1, 2}),
+			},
+			Events: []tracesdk.Event{
+				{
+					Time: time.Date(2020, time.March, 11, 19, 24, 30, 0, time.UTC),
+					Name: "ev1",
+					Attributes: []attribute.KeyValue{
+						attribute.Int64("eventattr1", 123),
+					},
+				},
+				{
+					Time:       time.Date(2020, time.March, 11, 19, 24, 45, 0, time.UTC),
+					Name:       "ev2",
+					Attributes: nil,
+				},
+			},
+			Status: tracesdk.Status{
+				Code:        codes.Ok,
+				Description: "",
+			},
+			Resource: res,
+		},
+		// typical span data with ERROR status
 		{
 			SpanContext: trace.NewSpanContext(trace.SpanContextConfig{
 				TraceID: trace.TraceID{0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F},
@@ -79,7 +160,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data with no parent (same as typical, but has
 		// invalid parent)
@@ -114,7 +195,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data of unspecified kind
 		{
@@ -152,7 +233,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data of internal kind
 		{
@@ -190,7 +271,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data of client kind
 		{
@@ -210,8 +291,8 @@ func TestModelConversion(t *testing.T) {
 				attribute.Int64("attr1", 42),
 				attribute.String("attr2", "bar"),
 				attribute.String("peer.hostname", "test-peer-hostname"),
-				attribute.String("net.peer.ip", "1.2.3.4"),
-				attribute.Int64("net.peer.port", 9876),
+				attribute.String("net.sock.peer.addr", "1.2.3.4"),
+				attribute.Int64("net.sock.peer.port", 9876),
 			},
 			Events: []tracesdk.Event{
 				{
@@ -231,7 +312,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data of producer kind
 		{
@@ -269,7 +350,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data of consumer kind
 		{
@@ -307,7 +388,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data with no events
 		{
@@ -332,7 +413,7 @@ func TestModelConversion(t *testing.T) {
 				Code:        codes.Error,
 				Description: "404, file not found",
 			},
-			Resource: resource,
+			Resource: res,
 		},
 		// span data with an "error" attribute set to "false"
 		{
@@ -365,12 +446,54 @@ func TestModelConversion(t *testing.T) {
 					Attributes: nil,
 				},
 			},
-			Resource: resource,
+			Resource: res,
 		},
 	}.Snapshots()
 
 	expectedOutputBatch := []zkmodel.SpanModel{
-		// model for typical span data
+		// model for typical span data with UNSET status
+		{
+			SpanContext: zkmodel.SpanContext{
+				TraceID: zkmodel.TraceID{
+					High: 0x001020304050607,
+					Low:  0x8090a0b0c0d0e0f,
+				},
+				ID:       zkmodel.ID(0xfffefdfcfbfaf9f8),
+				ParentID: zkmodelIDPtr(0x3f3e3d3c3b3a3938),
+				Debug:    false,
+				Sampled:  nil,
+				Err:      nil,
+			},
+			Name:      "foo",
+			Kind:      "SERVER",
+			Timestamp: time.Date(2020, time.March, 11, 19, 24, 0, 0, time.UTC),
+			Duration:  time.Minute,
+			Shared:    false,
+			LocalEndpoint: &zkmodel.Endpoint{
+				ServiceName: "model-test",
+			},
+			RemoteEndpoint: nil,
+			Annotations: []zkmodel.Annotation{
+				{
+					Timestamp: time.Date(2020, time.March, 11, 19, 24, 30, 0, time.UTC),
+					Value:     `ev1: {"eventattr1":123}`,
+				},
+				{
+					Timestamp: time.Date(2020, time.March, 11, 19, 24, 45, 0, time.UTC),
+					Value:     "ev2",
+				},
+			},
+			Tags: map[string]string{
+				"attr1":           "42",
+				"attr2":           "bar",
+				"attr3":           "[0,1,2]",
+				"service.name":    "model-test",
+				"service.version": "0.1.0",
+				"resource-attr1":  "42",
+				"resource-attr2":  "[0,1,2]",
+			},
+		},
+		// model for typical span data with OK status
 		{
 			SpanContext: zkmodel.SpanContext{
 				TraceID: zkmodel.TraceID{
@@ -406,8 +529,55 @@ func TestModelConversion(t *testing.T) {
 				"attr1":            "42",
 				"attr2":            "bar",
 				"attr3":            "[0,1,2]",
-				"otel.status_code": "Error",
+				"otel.status_code": "OK",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
+			},
+		},
+		// model for typical span data with ERROR status
+		{
+			SpanContext: zkmodel.SpanContext{
+				TraceID: zkmodel.TraceID{
+					High: 0x001020304050607,
+					Low:  0x8090a0b0c0d0e0f,
+				},
+				ID:       zkmodel.ID(0xfffefdfcfbfaf9f8),
+				ParentID: zkmodelIDPtr(0x3f3e3d3c3b3a3938),
+				Debug:    false,
+				Sampled:  nil,
+				Err:      nil,
+			},
+			Name:      "foo",
+			Kind:      "SERVER",
+			Timestamp: time.Date(2020, time.March, 11, 19, 24, 0, 0, time.UTC),
+			Duration:  time.Minute,
+			Shared:    false,
+			LocalEndpoint: &zkmodel.Endpoint{
+				ServiceName: "model-test",
+			},
+			RemoteEndpoint: nil,
+			Annotations: []zkmodel.Annotation{
+				{
+					Timestamp: time.Date(2020, time.March, 11, 19, 24, 30, 0, time.UTC),
+					Value:     `ev1: {"eventattr1":123}`,
+				},
+				{
+					Timestamp: time.Date(2020, time.March, 11, 19, 24, 45, 0, time.UTC),
+					Value:     "ev2",
+				},
+			},
+			Tags: map[string]string{
+				"attr1":            "42",
+				"attr2":            "bar",
+				"attr3":            "[0,1,2]",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data with no parent
@@ -445,8 +615,12 @@ func TestModelConversion(t *testing.T) {
 			Tags: map[string]string{
 				"attr1":            "42",
 				"attr2":            "bar",
-				"otel.status_code": "Error",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data of unspecified kind
@@ -484,8 +658,12 @@ func TestModelConversion(t *testing.T) {
 			Tags: map[string]string{
 				"attr1":            "42",
 				"attr2":            "bar",
-				"otel.status_code": "Error",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data of internal kind
@@ -523,8 +701,12 @@ func TestModelConversion(t *testing.T) {
 			Tags: map[string]string{
 				"attr1":            "42",
 				"attr2":            "bar",
-				"otel.status_code": "Error",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data of client kind
@@ -563,13 +745,17 @@ func TestModelConversion(t *testing.T) {
 				},
 			},
 			Tags: map[string]string{
-				"attr1":            "42",
-				"attr2":            "bar",
-				"net.peer.ip":      "1.2.3.4",
-				"net.peer.port":    "9876",
-				"peer.hostname":    "test-peer-hostname",
-				"otel.status_code": "Error",
-				"error":            "404, file not found",
+				"attr1":              "42",
+				"attr2":              "bar",
+				"net.sock.peer.addr": "1.2.3.4",
+				"net.sock.peer.port": "9876",
+				"peer.hostname":      "test-peer-hostname",
+				"otel.status_code":   "ERROR",
+				"error":              "404, file not found",
+				"service.name":       "model-test",
+				"service.version":    "0.1.0",
+				"resource-attr1":     "42",
+				"resource-attr2":     "[0,1,2]",
 			},
 		},
 		// model for span data of producer kind
@@ -607,8 +793,12 @@ func TestModelConversion(t *testing.T) {
 			Tags: map[string]string{
 				"attr1":            "42",
 				"attr2":            "bar",
-				"otel.status_code": "Error",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data of consumer kind
@@ -646,8 +836,12 @@ func TestModelConversion(t *testing.T) {
 			Tags: map[string]string{
 				"attr1":            "42",
 				"attr2":            "bar",
-				"otel.status_code": "Error",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data with no events
@@ -676,8 +870,12 @@ func TestModelConversion(t *testing.T) {
 			Tags: map[string]string{
 				"attr1":            "42",
 				"attr2":            "bar",
-				"otel.status_code": "Error",
+				"otel.status_code": "ERROR",
 				"error":            "404, file not found",
+				"service.name":     "model-test",
+				"service.version":  "0.1.0",
+				"resource-attr1":   "42",
+				"resource-attr2":   "[0,1,2]",
 			},
 		},
 		// model for span data with an "error" attribute set to "false"
@@ -712,7 +910,12 @@ func TestModelConversion(t *testing.T) {
 					Value:     "ev2",
 				},
 			},
-			Tags: nil, // should be omitted
+			Tags: map[string]string{
+				"service.name":    "model-test",
+				"service.version": "0.1.0",
+				"resource-attr1":  "42",
+				"resource-attr2":  "[0,1,2]",
+			}, // only resource tags should be included
 		},
 	}
 	gottenOutputBatch := SpanModels(inputBatch)
@@ -769,7 +972,40 @@ func TestTagsTransformation(t *testing.T) {
 			want: nil,
 		},
 		{
-			name: "statusCode",
+			name: "statusCode UNSET",
+			data: tracetest.SpanStub{
+				Attributes: []attribute.KeyValue{
+					attribute.String("key", keyValue),
+				},
+				Status: tracesdk.Status{
+					Code:        codes.Unset,
+					Description: "",
+				},
+			},
+			want: map[string]string{
+				"key": keyValue,
+			},
+		},
+		{
+			name: "statusCode OK",
+			data: tracetest.SpanStub{
+				Attributes: []attribute.KeyValue{
+					attribute.String("key", keyValue),
+					attribute.Bool("ok", true),
+				},
+				Status: tracesdk.Status{
+					Code:        codes.Ok,
+					Description: "",
+				},
+			},
+			want: map[string]string{
+				"key":              keyValue,
+				"ok":               "true",
+				"otel.status_code": "OK",
+			},
+		},
+		{
+			name: "statusCode ERROR",
 			data: tracetest.SpanStub{
 				Attributes: []attribute.KeyValue{
 					attribute.String("key", keyValue),
@@ -783,7 +1019,7 @@ func TestTagsTransformation(t *testing.T) {
 			want: map[string]string{
 				"error":            statusMessage,
 				"key":              keyValue,
-				"otel.status_code": codes.Error.String(),
+				"otel.status_code": "ERROR",
 			},
 		},
 		{
@@ -859,9 +1095,9 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 			data: tracetest.SpanStub{
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
-					semconv.PeerServiceKey.String("peer-service-test"),
-					semconv.NetPeerNameKey.String("peer-name-test"),
-					semconv.HTTPHostKey.String("http-host-test"),
+					semconv.PeerService("peer-service-test"),
+					semconv.NetPeerName("peer-name-test"),
+					semconv.NetSockPeerName("net-sock-peer-test"),
 				},
 			},
 			want: &zkmodel.Endpoint{
@@ -869,16 +1105,16 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 			},
 		},
 		{
-			name: "http-host-rank",
+			name: "net-sock-peer-rank",
 			data: tracetest.SpanStub{
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
-					semconv.HTTPHostKey.String("http-host-test"),
-					semconv.DBNameKey.String("db-name-test"),
+					semconv.NetSockPeerName("net-sock-peer-test"),
+					semconv.DBName("db-name-test"),
 				},
 			},
 			want: &zkmodel.Endpoint{
-				ServiceName: "http-host-test",
+				ServiceName: "net-sock-peer-test",
 			},
 		},
 		{
@@ -887,7 +1123,7 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
 					attribute.String("foo", "bar"),
-					semconv.DBNameKey.String("db-name-test"),
+					semconv.DBName("db-name-test"),
 				},
 			},
 			want: &zkmodel.Endpoint{
@@ -901,8 +1137,7 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 				Attributes: []attribute.KeyValue{
 					keyPeerHostname.String("peer-hostname-test"),
 					keyPeerAddress.String("peer-address-test"),
-					semconv.HTTPHostKey.String("http-host-test"),
-					semconv.DBNameKey.String("http-host-test"),
+					semconv.DBName("http-host-test"),
 				},
 			},
 			want: &zkmodel.Endpoint{
@@ -915,8 +1150,7 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
 					keyPeerAddress.String("peer-address-test"),
-					semconv.HTTPHostKey.String("http-host-test"),
-					semconv.DBNameKey.String("http-host-test"),
+					semconv.DBName("http-host-test"),
 				},
 			},
 			want: &zkmodel.Endpoint{
@@ -928,7 +1162,7 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 			data: tracetest.SpanStub{
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
-					semconv.NetPeerIPKey.String("INVALID"),
+					semconv.NetSockPeerAddr("INVALID"),
 				},
 			},
 			want: nil,
@@ -938,7 +1172,7 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 			data: tracetest.SpanStub{
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
-					semconv.NetPeerIPKey.String("0:0:1:5ee:bad:c0de:0:0"),
+					semconv.NetSockPeerAddr("0:0:1:5ee:bad:c0de:0:0"),
 				},
 			},
 			want: &zkmodel.Endpoint{
@@ -950,8 +1184,8 @@ func TestRemoteEndpointTransformation(t *testing.T) {
 			data: tracetest.SpanStub{
 				SpanKind: trace.SpanKindProducer,
 				Attributes: []attribute.KeyValue{
-					semconv.NetPeerIPKey.String("1.2.3.4"),
-					semconv.NetPeerPortKey.Int(9876),
+					semconv.NetSockPeerAddr("1.2.3.4"),
+					semconv.NetSockPeerPort(9876),
 				},
 			},
 			want: &zkmodel.Endpoint{
@@ -977,6 +1211,6 @@ func TestServiceName(t *testing.T) {
 	attrs = append(attrs, attribute.String("test_key", "test_value"))
 	assert.Equal(t, defaultServiceName, getServiceName(attrs))
 
-	attrs = append(attrs, semconv.ServiceNameKey.String("my_service"))
+	attrs = append(attrs, semconv.ServiceName("my_service"))
 	assert.Equal(t, "my_service", getServiceName(attrs))
 }
