@@ -15,7 +15,7 @@
 //go:build tools
 // +build tools
 
-package tools
+package tools // import "go.opentelemetry.io/otel/internal/tools"
 
 import (
 	_ "github.com/client9/misspell/cmd/misspell"
@@ -24,6 +24,8 @@ import (
 	_ "github.com/itchyny/gojq"
 	_ "github.com/jcchavezs/porto/cmd/porto"
 	_ "github.com/wadey/gocovmerge"
+	_ "go.opentelemetry.io/build-tools/crosslink"
+	_ "go.opentelemetry.io/build-tools/dbotconf"
 	_ "go.opentelemetry.io/build-tools/multimod"
 	_ "go.opentelemetry.io/build-tools/semconvgen"
 	_ "golang.org/x/tools/cmd/stringer"
