@@ -26,20 +26,18 @@ import (
 	"testing"
 	"time"
 
-	"go.opentelemetry.io/otel"
-	"go.opentelemetry.io/otel/attribute"
-	"go.opentelemetry.io/otel/codes"
-	semconv "go.opentelemetry.io/otel/semconv/v1.7.0"
-	"go.opentelemetry.io/otel/trace"
-
 	"github.com/google/go-cmp/cmp"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
+	"go.opentelemetry.io/otel"
+	"go.opentelemetry.io/otel/attribute"
+	"go.opentelemetry.io/otel/codes"
 	ottest "go.opentelemetry.io/otel/internal/internaltest"
-
 	"go.opentelemetry.io/otel/sdk/instrumentation"
 	"go.opentelemetry.io/otel/sdk/resource"
+	semconv "go.opentelemetry.io/otel/semconv/v1.17.0"
+	"go.opentelemetry.io/otel/trace"
 )
 
 const envVar = "OTEL_RESOURCE_ATTRIBUTES"
@@ -225,8 +223,8 @@ func TestSpanIsRecording(t *testing.T) {
 		} {
 			tp := NewTracerProvider(WithSampler(tc.sampler))
 			_, span := tp.Tracer(name).Start(context.Background(), "StartSpan")
-			defer span.End()
 			got := span.IsRecording()
+			span.End()
 			assert.Equal(t, got, tc.want, name)
 		}
 	})
@@ -365,6 +363,16 @@ func TestStartSpanWithParent(t *testing.T) {
 	}
 }
 
+// Test we get a successful span as a new root if a nil context is sent in, as opposed to a panic.
+// See https://github.com/open-telemetry/opentelemetry-go/issues/3109
+func TestStartSpanWithNilContext(t *testing.T) {
+	tp := NewTracerProvider()
+	tr := tp.Tracer("NoPanic")
+
+	// nolint:staticcheck // no nil context, but that's the point of the test.
+	assert.NotPanics(t, func() { tr.Start(nil, "should-not-panic") })
+}
+
 func TestStartSpanNewRootNotSampled(t *testing.T) {
 	alwaysSampleTp := NewTracerProvider()
 	sampledTr := alwaysSampleTp.Tracer("AlwaysSampled")
@@ -413,50 +421,22 @@ func TestSetSpanAttributesOnStart(t *testing.T) {
 			attribute.String("key1", "value1"),
 			attribute.String("key2", "value2"),
 		},
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "StartSpanAttribute"},
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "StartSpanAttribute"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("SetSpanAttributesOnStart: -got +want %s", diff)
 	}
 }
 
-func TestSetSpanAttributes(t *testing.T) {
-	te := NewTestExporter()
-	tp := NewTracerProvider(WithSyncer(te), WithResource(resource.Empty()))
-	span := startSpan(tp, "SpanAttribute")
-	span.SetAttributes(attribute.String("key1", "value1"))
-	got, err := endSpan(te, span)
-	if err != nil {
-		t.Fatal(err)
-	}
-
-	want := &snapshot{
-		spanContext: trace.NewSpanContext(trace.SpanContextConfig{
-			TraceID:    tid,
-			TraceFlags: 0x1,
-		}),
-		parent: sc.WithRemote(true),
-		name:   "span0",
-		attributes: []attribute.KeyValue{
-			attribute.String("key1", "value1"),
-		},
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "SpanAttribute"},
-	}
-	if diff := cmpDiff(got, want); diff != "" {
-		t.Errorf("SetSpanAttributes: -got +want %s", diff)
-	}
-}
-
 func TestSamplerAttributesLocalChildSpan(t *testing.T) {
 	sampler := &testSampler{prefix: "span", t: t}
 	te := NewTestExporter()
 	tp := NewTracerProvider(WithSampler(sampler), WithSyncer(te), WithResource(resource.Empty()))
 
 	ctx := context.Background()
-	ctx, span := startLocalSpan(tp, ctx, "SpanOne", "span0")
-	_, spanTwo := startLocalSpan(tp, ctx, "SpanTwo", "span1")
+	ctx, span := startLocalSpan(ctx, tp, "SpanOne", "span0")
+	_, spanTwo := startLocalSpan(ctx, tp, "SpanTwo", "span1")
 
 	spanTwo.End()
 	span.End()
@@ -471,72 +451,192 @@ func TestSamplerAttributesLocalChildSpan(t *testing.T) {
 	assert.Equal(t, []attribute.KeyValue{attribute.Int("callCount", 1)}, gotSpan1.Attributes())
 }
 
-func TestSetSpanAttributesOverLimit(t *testing.T) {
-	te := NewTestExporter()
-	tp := NewTracerProvider(WithSpanLimits(SpanLimits{AttributeCountLimit: 2}), WithSyncer(te), WithResource(resource.Empty()))
-
-	span := startSpan(tp, "SpanAttributesOverLimit")
-	span.SetAttributes(
-		attribute.Bool("key1", true),
+func TestSpanSetAttributes(t *testing.T) {
+	attrs := [...]attribute.KeyValue{
+		attribute.String("key1", "value1"),
 		attribute.String("key2", "value2"),
-		attribute.Bool("key1", false), // Replace key1.
-		attribute.Int64("key4", 4),    // Remove key2 and add key4
-	)
-	got, err := endSpan(te, span)
-	if err != nil {
-		t.Fatal(err)
+		attribute.String("key3", "value3"),
+		attribute.String("key4", "value4"),
+		attribute.String("key1", "value5"),
+		attribute.String("key2", "value6"),
+		attribute.String("key3", "value7"),
 	}
+	invalid := attribute.KeyValue{}
 
-	want := &snapshot{
-		spanContext: trace.NewSpanContext(trace.SpanContextConfig{
-			TraceID:    tid,
-			TraceFlags: 0x1,
-		}),
-		parent: sc.WithRemote(true),
-		name:   "span0",
-		attributes: []attribute.KeyValue{
-			attribute.Bool("key1", false),
-			attribute.Int64("key4", 4),
+	tests := []struct {
+		name        string
+		input       [][]attribute.KeyValue
+		wantAttrs   []attribute.KeyValue
+		wantDropped int
+	}{
+		{
+			name:      "array",
+			input:     [][]attribute.KeyValue{attrs[:3]},
+			wantAttrs: attrs[:3],
 		},
-		spanKind:               trace.SpanKindInternal,
-		droppedAttributeCount:  1,
-		instrumentationLibrary: instrumentation.Library{Name: "SpanAttributesOverLimit"},
-	}
-	if diff := cmpDiff(got, want); diff != "" {
-		t.Errorf("SetSpanAttributesOverLimit: -got +want %s", diff)
-	}
-}
+		{
+			name:      "single_value:array",
+			input:     [][]attribute.KeyValue{attrs[:1], attrs[1:3]},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name:      "array:single_value",
+			input:     [][]attribute.KeyValue{attrs[:2], attrs[2:3]},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name:      "single_values",
+			input:     [][]attribute.KeyValue{attrs[:1], attrs[1:2], attrs[2:3]},
+			wantAttrs: attrs[:3],
+		},
 
-func TestSetSpanAttributesWithInvalidKey(t *testing.T) {
-	te := NewTestExporter()
-	tp := NewTracerProvider(WithSpanLimits(SpanLimits{}), WithSyncer(te), WithResource(resource.Empty()))
+		// The tracing specification states:
+		//
+		//   For each unique attribute key, addition of which would result in
+		//   exceeding the limit, SDK MUST discard that key/value pair
+		//
+		// Therefore, adding attributes after the capacity is reached should
+		// result in those attributes being dropped.
 
-	span := startSpan(tp, "SpanToSetInvalidKeyOrValue")
-	span.SetAttributes(
-		attribute.Bool("", true),
-		attribute.Bool("key1", false),
-	)
-	got, err := endSpan(te, span)
-	if err != nil {
-		t.Fatal(err)
-	}
+		{
+			name:        "drop_last_added",
+			input:       [][]attribute.KeyValue{attrs[:3], attrs[3:4], attrs[3:4]},
+			wantAttrs:   attrs[:3],
+			wantDropped: 2,
+		},
 
-	want := &snapshot{
-		spanContext: trace.NewSpanContext(trace.SpanContextConfig{
-			TraceID:    tid,
-			TraceFlags: 0x1,
-		}),
-		parent: sc.WithRemote(true),
-		name:   "span0",
-		attributes: []attribute.KeyValue{
-			attribute.Bool("key1", false),
+		// The tracing specification states:
+		//
+		//   Setting an attribute with the same key as an existing attribute
+		//   SHOULD overwrite the existing attribute's value.
+		//
+		// Therefore, attributes are updated regardless of capacity state.
+
+		{
+			name:      "single_value_update",
+			input:     [][]attribute.KeyValue{attrs[:1], attrs[:3]},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name:      "all_update",
+			input:     [][]attribute.KeyValue{attrs[:3], attrs[4:7]},
+			wantAttrs: attrs[4:7],
+		},
+		{
+			name:      "all_update/multi",
+			input:     [][]attribute.KeyValue{attrs[:3], attrs[4:7], attrs[:3]},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name:      "deduplicate/under_capacity",
+			input:     [][]attribute.KeyValue{attrs[:1], attrs[:1], attrs[:1]},
+			wantAttrs: attrs[:1],
+		},
+		{
+			name:      "deduplicate/over_capacity",
+			input:     [][]attribute.KeyValue{attrs[:1], attrs[:1], attrs[:1], attrs[:3]},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name: "deduplicate/added",
+			input: [][]attribute.KeyValue{
+				attrs[:2],
+				{attrs[2], attrs[2], attrs[2]},
+			},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name: "deduplicate/added_at_cappacity",
+			input: [][]attribute.KeyValue{
+				attrs[:3],
+				{attrs[2], attrs[2], attrs[2]},
+			},
+			wantAttrs: attrs[:3],
+		},
+		{
+			name: "invalid",
+			input: [][]attribute.KeyValue{
+				{invalid},
+			},
+			wantDropped: 1,
+		},
+		{
+			name: "invalid_with_valid",
+			input: [][]attribute.KeyValue{
+				{invalid, attrs[0]},
+			},
+			wantAttrs:   attrs[:1],
+			wantDropped: 1,
+		},
+		{
+			name: "invalid_over_capacity",
+			input: [][]attribute.KeyValue{
+				{invalid, invalid, invalid, invalid, attrs[0]},
+			},
+			wantAttrs:   attrs[:1],
+			wantDropped: 4,
+		},
+		{
+			name: "valid:invalid/under_capacity",
+			input: [][]attribute.KeyValue{
+				attrs[:1],
+				{invalid},
+			},
+			wantAttrs:   attrs[:1],
+			wantDropped: 1,
+		},
+		{
+			name: "valid:invalid/over_capacity",
+			input: [][]attribute.KeyValue{
+				attrs[:1],
+				{invalid, invalid, invalid, invalid},
+			},
+			wantAttrs:   attrs[:1],
+			wantDropped: 4,
+		},
+		{
+			name: "valid_at_capacity:invalid",
+			input: [][]attribute.KeyValue{
+				attrs[:3],
+				{invalid, invalid, invalid, invalid},
+			},
+			wantAttrs:   attrs[:3],
+			wantDropped: 4,
 		},
-		spanKind:               trace.SpanKindInternal,
-		droppedAttributeCount:  0,
-		instrumentationLibrary: instrumentation.Library{Name: "SpanToSetInvalidKeyOrValue"},
 	}
-	if diff := cmpDiff(got, want); diff != "" {
-		t.Errorf("SetSpanAttributesWithInvalidKey: -got +want %s", diff)
+
+	const (
+		capacity = 3
+		instName = "TestSpanAttributeCapacity"
+		spanName = "test span"
+	)
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			te := NewTestExporter()
+			sl := NewSpanLimits()
+			sl.AttributeCountLimit = capacity
+			tp := NewTracerProvider(WithSyncer(te), WithSpanLimits(sl))
+			_, span := tp.Tracer(instName).Start(context.Background(), spanName)
+			for _, a := range test.input {
+				span.SetAttributes(a...)
+			}
+			span.End()
+
+			require.Implements(t, (*ReadOnlySpan)(nil), span)
+			roSpan := span.(ReadOnlySpan)
+
+			// Ensure the span itself is valid.
+			assert.ElementsMatch(t, test.wantAttrs, roSpan.Attributes(), "expected attributes")
+			assert.Equal(t, test.wantDropped, roSpan.DroppedAttributes(), "dropped attributes")
+
+			snap, ok := te.GetSpan(spanName)
+			require.Truef(t, ok, "span %s not exported", spanName)
+
+			// Ensure the exported span snapshot is valid.
+			assert.ElementsMatch(t, test.wantAttrs, snap.Attributes(), "expected attributes")
+			assert.Equal(t, test.wantDropped, snap.DroppedAttributes(), "dropped attributes")
+		})
 	}
 }
 
@@ -576,8 +676,8 @@ func TestEvents(t *testing.T) {
 			{Name: "foo", Attributes: []attribute.KeyValue{k1v1}},
 			{Name: "bar", Attributes: []attribute.KeyValue{k2v2, k3v3}},
 		},
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "Events"},
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "Events"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("Message Events: -got +want %s", diff)
@@ -586,7 +686,9 @@ func TestEvents(t *testing.T) {
 
 func TestEventsOverLimit(t *testing.T) {
 	te := NewTestExporter()
-	tp := NewTracerProvider(WithSpanLimits(SpanLimits{EventCountLimit: 2}), WithSyncer(te), WithResource(resource.Empty()))
+	sl := NewSpanLimits()
+	sl.EventCountLimit = 2
+	tp := NewTracerProvider(WithSpanLimits(sl), WithSyncer(te), WithResource(resource.Empty()))
 
 	span := startSpan(tp, "EventsOverLimit")
 	k1v1 := attribute.String("key1", "value1")
@@ -625,9 +727,9 @@ func TestEventsOverLimit(t *testing.T) {
 			{Name: "foo", Attributes: []attribute.KeyValue{k1v1}},
 			{Name: "bar", Attributes: []attribute.KeyValue{k2v2, k3v3}},
 		},
-		droppedEventCount:      2,
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "EventsOverLimit"},
+		droppedEventCount:    2,
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "EventsOverLimit"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("Message Event over limit: -got +want %s", diff)
@@ -661,11 +763,11 @@ func TestLinks(t *testing.T) {
 			TraceID:    tid,
 			TraceFlags: 0x1,
 		}),
-		parent:                 sc.WithRemote(true),
-		name:                   "span0",
-		links:                  []Link{{l1.SpanContext, l1.Attributes, 0}, {l2.SpanContext, l2.Attributes, 0}},
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "Links"},
+		parent:               sc.WithRemote(true),
+		name:                 "span0",
+		links:                []Link{{l1.SpanContext, l1.Attributes, 0}, {l2.SpanContext, l2.Attributes, 0}},
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "Links"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("Link: -got +want %s", diff)
@@ -688,7 +790,9 @@ func TestLinksOverLimit(t *testing.T) {
 	sc2 := trace.NewSpanContext(trace.SpanContextConfig{TraceID: trace.TraceID([16]byte{1, 1}), SpanID: trace.SpanID{3}})
 	sc3 := trace.NewSpanContext(trace.SpanContextConfig{TraceID: trace.TraceID([16]byte{1, 1}), SpanID: trace.SpanID{3}})
 
-	tp := NewTracerProvider(WithSpanLimits(SpanLimits{LinkCountLimit: 2}), WithSyncer(te), WithResource(resource.Empty()))
+	sl := NewSpanLimits()
+	sl.LinkCountLimit = 2
+	tp := NewTracerProvider(WithSpanLimits(sl), WithSyncer(te), WithResource(resource.Empty()))
 
 	span := startSpan(tp, "LinksOverLimit",
 		trace.WithLinks(
@@ -717,9 +821,9 @@ func TestLinksOverLimit(t *testing.T) {
 			{SpanContext: sc2, Attributes: []attribute.KeyValue{k2v2}, DroppedAttributeCount: 0},
 			{SpanContext: sc3, Attributes: []attribute.KeyValue{k3v3}, DroppedAttributeCount: 0},
 		},
-		droppedLinkCount:       1,
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "LinksOverLimit"},
+		droppedLinkCount:     1,
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "LinksOverLimit"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("Link over limit: -got +want %s", diff)
@@ -767,7 +871,7 @@ func TestSetSpanStatus(t *testing.T) {
 			Code:        codes.Error,
 			Description: "Error",
 		},
-		instrumentationLibrary: instrumentation.Library{Name: "SpanStatus"},
+		instrumentationScope: instrumentation.Scope{Name: "SpanStatus"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("SetSpanStatus: -got +want %s", diff)
@@ -797,7 +901,7 @@ func TestSetSpanStatusWithoutMessageWhenStatusIsNotError(t *testing.T) {
 			Code:        codes.Ok,
 			Description: "",
 		},
-		instrumentationLibrary: instrumentation.Library{Name: "SpanStatus"},
+		instrumentationScope: instrumentation.Scope{Name: "SpanStatus"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("SetSpanStatus: -got +want %s", diff)
@@ -856,7 +960,7 @@ func startNamedSpan(tp *TracerProvider, trName, name string, args ...trace.SpanS
 // passed name and with the passed context. The context is returned
 // along with the span so this parent can be used to create child
 // spans.
-func startLocalSpan(tp *TracerProvider, ctx context.Context, trName, name string, args ...trace.SpanStartOption) (context.Context, trace.Span) {
+func startLocalSpan(ctx context.Context, tp *TracerProvider, trName, name string, args ...trace.SpanStartOption) (context.Context, trace.Span) {
 	ctx, span := tp.Tracer(trName).Start(
 		ctx,
 		name,
@@ -876,10 +980,10 @@ func startLocalSpan(tp *TracerProvider, ctx context.Context, trName, name string
 // It also clears spanID in the to make the comparison easier.
 func endSpan(te *testExporter, span trace.Span) (*snapshot, error) {
 	if !span.IsRecording() {
-		return nil, fmt.Errorf("IsRecording: got false, want true")
+		return nil, fmt.Errorf("method IsRecording: got false, want true")
 	}
 	if !span.SpanContext().IsSampled() {
-		return nil, fmt.Errorf("IsSampled: got false, want true")
+		return nil, fmt.Errorf("method IsSampled: got false, want true")
 	}
 	span.End()
 	if te.Len() != 1 {
@@ -1078,10 +1182,10 @@ func TestCustomStartEndTime(t *testing.T) {
 		t.Fatalf("got %d exported spans, want one span", te.Len())
 	}
 	got := te.Spans()[0]
-	if got.StartTime() != startTime {
+	if !got.StartTime().Equal(startTime) {
 		t.Errorf("expected start time to be %s, got %s", startTime, got.StartTime())
 	}
-	if got.EndTime() != endTime {
+	if !got.EndTime().Equal(endTime) {
 		t.Errorf("expected end time to be %s, got %s", endTime, got.EndTime())
 	}
 }
@@ -1131,12 +1235,12 @@ func TestRecordError(t *testing.T) {
 					Name: semconv.ExceptionEventName,
 					Time: errTime,
 					Attributes: []attribute.KeyValue{
-						semconv.ExceptionTypeKey.String(s.typ),
-						semconv.ExceptionMessageKey.String(s.msg),
+						semconv.ExceptionType(s.typ),
+						semconv.ExceptionMessage(s.msg),
 					},
 				},
 			},
-			instrumentationLibrary: instrumentation.Library{Name: "RecordError"},
+			instrumentationScope: instrumentation.Scope{Name: "RecordError"},
 		}
 		if diff := cmpDiff(got, want); diff != "" {
 			t.Errorf("SpanErrorOptions: -got +want %s", diff)
@@ -1175,12 +1279,12 @@ func TestRecordErrorWithStackTrace(t *testing.T) {
 				Name: semconv.ExceptionEventName,
 				Time: errTime,
 				Attributes: []attribute.KeyValue{
-					semconv.ExceptionTypeKey.String(typ),
-					semconv.ExceptionMessageKey.String(msg),
+					semconv.ExceptionType(typ),
+					semconv.ExceptionMessage(msg),
 				},
 			},
 		},
-		instrumentationLibrary: instrumentation.Library{Name: "RecordError"},
+		instrumentationScope: instrumentation.Scope{Name: "RecordError"},
 	}
 
 	assert.Equal(t, got.spanContext, want.spanContext)
@@ -1220,7 +1324,7 @@ func TestRecordErrorNil(t *testing.T) {
 			Code:        codes.Unset,
 			Description: "",
 		},
-		instrumentationLibrary: instrumentation.Library{Name: "RecordErrorNil"},
+		instrumentationScope: instrumentation.Scope{Name: "RecordErrorNil"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("SpanErrorOptions: -got +want %s", diff)
@@ -1334,9 +1438,9 @@ func TestWithResource(t *testing.T) {
 				attributes: []attribute.KeyValue{
 					attribute.String("key1", "value1"),
 				},
-				spanKind:               trace.SpanKindInternal,
-				resource:               tc.want,
-				instrumentationLibrary: instrumentation.Library{Name: "WithResource"},
+				spanKind:             trace.SpanKindInternal,
+				resource:             tc.want,
+				instrumentationScope: instrumentation.Scope{Name: "WithResource"},
 			}
 			if diff := cmpDiff(got, want); diff != "" {
 				t.Errorf("WithResource:\n  -got +want %s", diff)
@@ -1369,7 +1473,7 @@ func TestWithInstrumentationVersionAndSchema(t *testing.T) {
 		parent:   sc.WithRemote(true),
 		name:     "span0",
 		spanKind: trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{
+		instrumentationScope: instrumentation.Scope{
 			Name:      "WithInstrumentationVersion",
 			Version:   "v0.1.0",
 			SchemaURL: "https://opentelemetry.io/schemas/1.2.0",
@@ -1398,8 +1502,8 @@ func TestSpanCapturesPanic(t *testing.T) {
 	require.Len(t, spans[0].Events(), 1)
 	assert.Equal(t, spans[0].Events()[0].Name, semconv.ExceptionEventName)
 	assert.Equal(t, spans[0].Events()[0].Attributes, []attribute.KeyValue{
-		semconv.ExceptionTypeKey.String("*errors.errorString"),
-		semconv.ExceptionMessageKey.String("error message"),
+		semconv.ExceptionType("*errors.errorString"),
+		semconv.ExceptionMessage("error message"),
 	})
 }
 
@@ -1478,6 +1582,8 @@ func TestReadOnlySpan(t *testing.T) {
 	assert.Equal(t, "", ro.Status().Description)
 	assert.Equal(t, "ReadOnlySpan", ro.InstrumentationLibrary().Name)
 	assert.Equal(t, "3", ro.InstrumentationLibrary().Version)
+	assert.Equal(t, "ReadOnlySpan", ro.InstrumentationScope().Name)
+	assert.Equal(t, "3", ro.InstrumentationScope().Version)
 	assert.Equal(t, kv.Key, ro.Resource().Attributes()[0].Key)
 	assert.Equal(t, kv.Value, ro.Resource().Attributes()[0].Value)
 
@@ -1536,7 +1642,6 @@ func TestReadWriteSpan(t *testing.T) {
 	// Verify the span can be written to.
 	rw.SetName("bar")
 	assert.Equal(t, "bar", rw.Name())
-
 	// NOTE: This function tests ReadWriteSpan which is an interface which
 	// embeds trace.Span and ReadOnlySpan. Since both of these interfaces have
 	// their own tests, there is no point in testing all the possible methods
@@ -1546,8 +1651,10 @@ func TestReadWriteSpan(t *testing.T) {
 
 func TestAddEventsWithMoreAttributesThanLimit(t *testing.T) {
 	te := NewTestExporter()
+	sl := NewSpanLimits()
+	sl.AttributePerEventCountLimit = 2
 	tp := NewTracerProvider(
-		WithSpanLimits(SpanLimits{AttributePerEventCountLimit: 2}),
+		WithSpanLimits(sl),
 		WithSyncer(te),
 		WithResource(resource.Empty()),
 	)
@@ -1600,8 +1707,8 @@ func TestAddEventsWithMoreAttributesThanLimit(t *testing.T) {
 				DroppedAttributeCount: 2,
 			},
 		},
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "AddSpanEventWithOverLimitedAttributes"},
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "AddSpanEventWithOverLimitedAttributes"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("SetSpanAttributesOverLimit: -got +want %s", diff)
@@ -1610,8 +1717,10 @@ func TestAddEventsWithMoreAttributesThanLimit(t *testing.T) {
 
 func TestAddLinksWithMoreAttributesThanLimit(t *testing.T) {
 	te := NewTestExporter()
+	sl := NewSpanLimits()
+	sl.AttributePerLinkCountLimit = 1
 	tp := NewTracerProvider(
-		WithSpanLimits(SpanLimits{AttributePerLinkCountLimit: 1}),
+		WithSpanLimits(sl),
 		WithSyncer(te),
 		WithResource(resource.Empty()),
 	)
@@ -1653,8 +1762,8 @@ func TestAddLinksWithMoreAttributesThanLimit(t *testing.T) {
 				DroppedAttributeCount: 2,
 			},
 		},
-		spanKind:               trace.SpanKindInternal,
-		instrumentationLibrary: instrumentation.Library{Name: "Links"},
+		spanKind:             trace.SpanKindInternal,
+		instrumentationScope: instrumentation.Scope{Name: "Links"},
 	}
 	if diff := cmpDiff(got, want); diff != "" {
 		t.Errorf("Link: -got +want %s", diff)
@@ -1679,7 +1788,7 @@ func (s stateSampler) Description() string {
 	return "stateSampler"
 }
 
-// Check that a new span propagates the SamplerResult.TraceState
+// Check that a new span propagates the SamplerResult.TraceState.
 func TestSamplerTraceState(t *testing.T) {
 	mustTS := func(ts trace.TraceState, err error) trace.TraceState {
 		require.NoError(t, err)
@@ -1804,7 +1913,6 @@ func TestSamplerTraceState(t *testing.T) {
 			}
 		})
 	}
-
 }
 
 type testIDGenerator struct {
@@ -1854,3 +1962,11 @@ func TestWithIDGenerator(t *testing.T) {
 		require.NoError(t, err)
 	}
 }
+
+func TestEmptyRecordingSpanAttributes(t *testing.T) {
+	assert.Nil(t, (&recordingSpan{}).Attributes())
+}
+
+func TestEmptyRecordingSpanDroppedAttributes(t *testing.T) {
+	assert.Equal(t, 0, (&recordingSpan{}).DroppedAttributes())
+}
